@@ -12,9 +12,6 @@ from abc import ABC, abstractmethod
 from typing import Tuple, Optional
 
 import pygame
-# I'm hoping you can init multiple times
-pygame.init()
-
 from custom_inherit import doc_inherit
 from pygame import Color
 
@@ -22,11 +19,15 @@ from pi_touch_gui._utilities import snapped_value_string
 from pi_touch_gui.colors import WHITE, BLACK, MEDIUM_GRAY
 from pi_touch_gui.multitouch.raspberrypi_ts import (TS_PRESS, TS_RELEASE,
                                                     TS_MOVE)
+from pi_touch_gui.widget_functions import ButtonFunction
 
 # Default font name - specify one here if you want something other
 # than the system default.
 FONT_NAME = None
 
+# I'm hoping you can init multiple times
+pygame.init()
+
 
 class IWidgetInterface(ABC):
     """ The bare minimum methods to implement to be a widget.
@@ -62,37 +63,12 @@ class IWidgetInterface(ABC):
         """
 
     @abstractmethod
-    def focus(self, enabled):
-        """ Set the widget's focus, for the gestural interface.
-
-        Parameters
-        ----------
-        enabled : bool
-            True if this Widget is receiving focus.
-        """
-
-    @abstractmethod
-    def can_focus(self):
-        """ Return True if this widget can accept focus.
-        """
-
-    @abstractmethod
-    def render(self, surface):
+    def render(self, page, surface):
         """ Render to the device pygame Surface.
 
         Parameters
         ----------
-        surface : pygame.Surface
-        """
-
-    @abstractmethod
-    def render_focus(self, surface):
-        """ Render a default focus marker that is the extent.
-
-        Only called if can_focus() returns True, but should be defined anyway.
-
-        Parameters
-        ----------
+        page : Page
         surface : pygame.Surface
         """
 
@@ -114,7 +90,6 @@ class IWidget(IWidgetInterface, ABC):
 
     def __init__(self, position, size,
                  widget_id=None,
-                 neighbors=None,
                  font_size=None,
                  label=None,
                  color1=None,
@@ -138,12 +113,6 @@ class IWidget(IWidgetInterface, ABC):
             extent to something less rectangular.
         widget_id : str
             Optional ID for this widget, if not given, gets a random uuid.
-        neighbors : List[Union[str, IWidget]]
-            The neighbors of this widget, specified by the widget itself or
-            that widget's ID.  The list order is [GO_LEFT, GO_RIGHT,
-            GO_UP, GO_DOWN].  If you want to stop at an edge, specify
-            the neighbor as Page.WALL. These neighbor hints help with
-            keyboard arrow navigation of the Page.
         font_size : int
             Height of the font in pixels.  If not specified, will be 1/2 the
             height of the widget.
@@ -184,11 +153,6 @@ class IWidget(IWidgetInterface, ABC):
         self._font = font
 
         self._pressed = False
-        self._focus = False
-        # We can't currently serialize neighbors, as it causes infinite loops
-        # Need to develop a non-object mechanism to name neighbors
-        self._neighbors = neighbors
-
         self._touches = []
 
         # Probe for the existence of the event handlers.
@@ -219,41 +183,6 @@ class IWidget(IWidgetInterface, ABC):
         """
         return len(self._touches) > 0 or self._pressed
 
-    def set_neighbors(self, new_neighbors):
-        """ For enhanced keyboard navigation, point to the neighboring widgets.
-
-        If neighbors are not specified, a more limited keyboard navigation is
-        available based on the list order of the widgets in the Page.
-
-        Note that you specify the neighbors as ID or IWidget objects, but on
-        use the system takes the liberty of changing the representation to
-        whatever it sees fit so don't be surprised if the internal values
-        change.
-
-        Parameters
-        ----------
-        new_neighbors : List(Union(String, IWidget))
-            The list of neighbors in order Up, Down, Left, Right; where each
-            entry can be an IWidget, the widget_id, or Page.WALL
-        """
-        if self._neighbors is None:
-            self._neighbors = [None, None, None, None]
-
-        for idx, neighbor in enumerate(new_neighbors):
-            if isinstance(neighbor, IWidget):
-                self._neighbors[idx] = neighbor
-            elif isinstance(neighbor, str):
-                self._neighbors[idx] = neighbor
-            else:
-                raise ValueError(f"Unsupported neighbor {idx} type "
-                                 f"{type(neighbor)!r}")
-
-    def focus(self, enabled):
-        self._focus = enabled
-
-    def can_focus(self) -> bool:
-        return True
-
     def touch_inside(self, touch) -> bool:
         """ Test a touch against the generic widget rectangle.
 
@@ -297,10 +226,10 @@ class IWidget(IWidgetInterface, ABC):
         # "Not Me" - report not consumed
         return False, None
 
-    def press(self):
+    def press(self, page):
         self._pressed = True
 
-    def release(self):
+    def release(self, page):
         self._pressed = False
 
     def state_colors(self, active=None) -> Tuple[Color, Color]:
@@ -326,15 +255,6 @@ class IWidget(IWidgetInterface, ABC):
             label_color = self.label_color1
         return color, label_color
 
-    def focus_color(self):
-        """ Return a changing color for the focus highlight
-        """
-        time = pygame.time.get_ticks()
-        pulse = math.sin(time / 133) + 1.0  # sets to range 0..2
-        return (pulse * 64 + 127,
-                64 - pulse * 16,
-                64 - pulse * 16)
-
     def render_centered_text(self, surface, text, color, bg_color=None):
         """ Render text in the center of the widget.
 
@@ -398,17 +318,6 @@ class IWidget(IWidgetInterface, ABC):
         textpos.centery = self._y + (self._h / 2)
         surface.blit(text_bmp, textpos)
 
-    def render_focus(self, surface):
-        """ Render a default focus outline.
-
-        Parameters
-        ----------
-        surface : pygame.Surface
-        """
-        fx, fy = self._x - 2, self._y - 2
-        fw, fh = self._w + 4, self._h + 4
-        pygame.draw.rect(surface, self.focus_color(), ((fx, fy), (fw, fh)), 4)
-
 
 class IButtonWidget(IWidget):
     """ ButtonWidget base that extends IWidget for push buttons.
@@ -419,12 +328,19 @@ class IButtonWidget(IWidget):
         """
         Parameters
         ----------
-        function : Callable[[IWidget, bool], Optional['Page']]
+        function : ButtonFunction
             Function to call on button events, with a True/False parameter
             that is True when pressed and False when released.
             Optionally returns a new Page to change pages.
         """
+        if (function is not None and
+                not issubclass(function.__class__, ButtonFunction)):
+            raise ValueError(f"Function type {type(function)} is not a "
+                             f"ButtonFunction.")
         self.function = function
+        # Override lets things like the ButtonGroup take over the function,
+        # but doesn't serialize so it won't pollute the serial form
+        self._function_override = None
         super(IButtonWidget, self).__init__(position, size, **kwargs)
 
     def _on_release(self, touch) -> Optional['Page']:
@@ -443,17 +359,19 @@ class IButtonWidget(IWidget):
         Page
             If specified, the new Page to go to.
         """
-        return self.release()
+        return self.release(touch.page)
 
-    def release(self):
+    def release(self, page):
         """ Trigger the button, calling its function if specified.
         """
-        super(IButtonWidget, self).release()
-        if self.function:
-            return self.function(self, False)
+        super(IButtonWidget, self).release(page)
+        if self._function_override:
+            return self._function_override.update(page, self, False)
+        elif self.function:
+            return self.function.update(page, self, False)
 
     @doc_inherit(IWidget.render, style='numpy_with_merge')
-    def render(self, surface):
+    def render(self, page, surface):
         """ Render the generic button as a filled rectangle with a label.
         """
         color, label_color = self.state_colors()
@@ -461,9 +379,6 @@ class IButtonWidget(IWidget):
         pygame.draw.rect(surface, color, (self.position, self.size), 0)
         self.render_centered_text(surface, self.label, label_color, color)
 
-        if self._focus:
-            self.render_focus(surface)
-
 
 class IControlWidget(IWidget, ABC):
     """ ControlWidget base that extends IWidget for input controls.
@@ -475,7 +390,6 @@ class IControlWidget(IWidget, ABC):
                  start_value=None,
                  snap_value=None,
                  dynamic=None,
-                 function=None,
                  **kwargs):
         """
         Parameters
@@ -492,11 +406,7 @@ class IControlWidget(IWidget, ABC):
         dynamic : bool
             True if the function is called each frame the control is active,
             otherwise the function is only called on release.
-        function : Callable[[IWidget, float], Optional['Page']]
-            Function to call on control events, with a new value for the
-            control. Optionally returns a new Page to change pages on release.
         """
-        self.function = function
         super(IControlWidget, self).__init__(position, size,
                                              **kwargs)
 
@@ -519,6 +429,9 @@ class IControlWidget(IWidget, ABC):
 
         self.dynamic = dynamic or False
 
+        # Function must be set by the child classes AFTER this init
+        self.function = None
+
     def map_value_to_adjusted_value(self, value) -> float:
         """ Map a value in 0..1 to an interpolated value in min_val..max_val.
 
@@ -557,7 +470,7 @@ class IControlWidget(IWidget, ABC):
         Returns
         -------
         str
-            The string version of self.adjusted_value, which is the value
+            The string version of self._adjusted_value, which is the value
             the widget represents to the outside world.
         """
         return snapped_value_string(self._adjusted_value, self.snap_value)
@@ -574,7 +487,7 @@ class IControlWidget(IWidget, ABC):
             Touch object; generally not used for controls
         """
         if self.dynamic is True and self.function is not None:
-            return self.function(self, self._adjusted_value)
+            return self.function.update(touch.page, self, self._adjusted_value)
 
     def _adjust(self, direction):
         """ Adjust the control up or down based on sign(direction)
@@ -589,7 +502,7 @@ class IControlWidget(IWidget, ABC):
         self._value = self.normalize_adjusted_value(value)
 
         if self.function is not None:
-            return self.function(self, self._adjusted_value)
+            return self.function.update(self, self._adjusted_value)
 
     def _on_release(self, touch):
         """ Controls call their function on the release event.
@@ -602,11 +515,11 @@ class IControlWidget(IWidget, ABC):
         touch : Touch
             Touch object; generally not used for controls
         """
-        self.release()
+        self.release(touch.page)
 
-    def release(self):
+    def release(self, page):
         """ Trigger the button, calling its function if specified with
         the adjusted value.
         """
         if self.function is not None:
-            return self.function(self, self._adjusted_value)
+            return self.function.update(page, self, self._adjusted_value)
